#!/usr/bin/python3
from argparse import Action
from requests.utils import default_user_agent
from random import choice
import functools
from tqdm import tqdm

"""A module for all the miscellaneous pieces of code that are needed,
   but would make the main module look a bit clunky and cluttered. This
   includes a list of user agents to choose at random, a specialized 
   commandline argparse Action, and print decorator for formated output."""

user_agent_list = ('Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36',
    'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:53.0) Gecko/20100101 Firefox/53.0',
    'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.79 Safari/537.36 Edge/14.14393',
    'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)', 'Mozilla/5.0 (Windows; U; MSIE 7.0; Windows NT 6.0; en-US)',
    'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)',
    'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0;  Trident/5.0)', 'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; MDDCJS)',
    'Mozilla/5.0 (compatible, MSIE 11, Windows NT 6.3; Trident/7.0; rv:11.0) like Gecko',
    'Mozilla/5.0 (iPad; CPU OS 8_4_1 like Mac OS X) AppleWebKit/600.1.4 (KHTML, like Gecko) Version/8.0 Mobile/12H321 Safari/600.1.4',
    'Mozilla/5.0 (iPhone; CPU iPhone OS 10_3_1 like Mac OS X) AppleWebKit/603.1.30 (KHTML, like Gecko) Version/10.0 Mobile/14E304 Safari/602.1',
    'Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)', 'Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm)',
    'Mozilla/5.0 (Linux; Android 6.0.1; SAMSUNG SM-G570Y Build/MMB29K) AppleWebKit/537.36 (KHTML, like Gecko) SamsungBrowser/4.0 Chrome/44.0.2403.133 Mobile Safari/537.36',
    'Mozilla/5.0 (Linux; Android 5.0; SAMSUNG SM-N900 Build/LRX21V) AppleWebKit/537.36 (KHTML, like Gecko) SamsungBrowser/2.1 Chrome/34.0.1847.76 Mobile Safari/537.36',
    'Mozilla/5.0 (Linux; Android 6.0.1; SAMSUNG SM-N910F Build/MMB29M) AppleWebKit/537.36 (KHTML, like Gecko) SamsungBrowser/4.0 Chrome/44.0.2403.133 Mobile Safari/537.36',
    'Mozilla/5.0 (Linux; U; Android-4.0.3; en-us; Galaxy Nexus Build/IML74K) AppleWebKit/535.7 (KHTML, like Gecko) CrMo/16.0.912.75 Mobile Safari/535.7',
    'Mozilla/5.0 (Linux; Android 7.0; HTC 10 Build/NRD90M) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.83 Mobile Safari/537.36', 'curl/7.35.0',
    'Wget/1.15 (linux-gnu)', 'Lynx/2.8.8pre.4 libwww-FM/2.14 SSL-MM/1.4.1 GNUTLS/2.12.23')

class UserAgentAction(Action):
    def __call__(self, parser, namespace, values=None, option_string=None):
        if values == None:
            agent = RandomAgent()
        else:
            agent = values
        setattr(namespace, self.dest, agent)

def RandomAgent():
    return choice(user_agent_list)

status_codes = [200,204,301,302,307,401,403]

def pretty(func):
    '''Decorator function for output formating.'''
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        func(*args, **kwargs)
        tqdm.write('=' * 65)
    return wrapper

def lister(exts):
    """Properly converts status codes or extenstions options
        into lists for latter use."""
    if isinstance(exts, str):
        exts = [i for i in exts.split(',')]
    elif isinstance(exts, list):
        pass
    else:
        if exts == None:
            return ['']
        else:
            raise ValueError(f"{exts} type {type(exts)} is not valid! Need string or list.")
    if any(map(lambda x: not x.isdigit(), exts)):
        if exts != ['']:
            exts = ['.'+i for i in exts]
            exts.insert(0,'')
    else:
        try:
            exts = [int(i) for i in exts]
        except ValueError:
            raise ValueError(f"Status code in {exts} is not a valid integer.")
    return exts


#!/usr/bin/python3
"""The main implementation file for pybuster.
   It contains the Pybuster class that is used
   to create and run the scanner"""
import requests, urllib.request
from requests_ntlm import HttpNtlmAuth
from random import shuffle, choice
import concurrent.futures
import argparse
from string import ascii_letters
import sys, time
from pydirbuster import utils, __version__
from pathlib import Path
from collections import defaultdict
from tqdm import tqdm

class Pybuster(object):
    """The Pybuster class is the main interface for this website scanner.

       Pybuster Class:

       param: url - The website base url for scanning.
       type: str

       param: wordfile - The filepath, relative or absolute for wordlist.
       type: str

       param: threads - The number of threads for the scanner to run. Default = 15
       type: int

       param: exts - The list of file extensions to check. default = ['']
       It is best pass it a list, ie. ['php', 'html', 'png'], but it can be
       passed a comma delimited string ex., 'php,html,png'
       type: list

       param: logfile - The name of an output file write results to.
       type: str

       param: codes - The http status codes to accept in responses.
       Can be passed a list of numbers in either int or str forms, or a 
       comma delimited string. So ['200','204','301','302','307','401','403'],
       [200,204,301,302,307,401,403], and "200,204,301,302,307,401,403" are all
       valid, but the inner values must be able to converted to integers.
       Default = [200,204,301,302,307,401,403]
       param: user - HTTP username - Default = None
       type: list

       param: password - HTTP password - Default = None
       type: str

       param: force - Switch for forcing processing on a wildcard matching webserver.
       To switch on this option use the boolean value True, but any valid truthy value
       would work. Default = False
       type: bool

       param: user_agent - The user agent to use in http requests. If None is
       passed as the user_agent, a randomizer will be called to supply one.
       Default is the current python requests user agent.
       type: str"""
    def __init__(self, url:str, wordfile:str, threads:int=15, exts:list=[''],
                 logfile:str=None, codes:list=utils.status_codes, user:str=None,
                 password:str=None, force:bool=False,
                 user_agent:str=requests.utils.default_user_agent()):
        file = Path(wordfile)
        if not file.exists():
            raise FileNotFoundError(f"{wordfile} doesn't exist")
        self.url = url if url.endswith('/') else url + '/'
        self.wordlist = ''
        self.wordfile = wordfile
        with open(wordfile) as f:
            self.wordlist = f.read().splitlines()
        self.threads = threads
        self.exts = self._lister(exts, 'ext')
        self.logfile = logfile
        self.codes = self._lister(codes, 'http')
        self.user = user
        self.password = password
        self.force = force
        self.user_agent = user_agent or utils.RandomAgent()
        self.results = defaultdict(list)

    def _lister(self, exts, ver:str):
        """Properly converts status codes or extenstions options
           into lists for latter use. ver is an internal parameter
           to differentiate what kind of list is needed to output
           since they can functionaly be the same on input."""
        if isinstance(exts, str):
            exts = [i for i in exts.split(',')]
        elif isinstance(exts, list):
            pass
        else:
            if exts == None:
                return ['']
            else:
                raise ValueError(f"{exts} type {type(exts)} is not valid! Need string or list.")
        if ver == 'ext':
            if exts != ['']:
                exts = ['.'+i if not i.startswith('.') else i for i in exts]
                exts.insert(0,'')
                try:
                    exts.remove('.')
                except ValueError:
                    pass
        else:
            try:
                exts = [int(i) for i in exts]
            except ValueError:
                raise ValueError(f"Status code in {exts} is not a valid integer.")
        return exts

    def _brute(self, client:requests.Session, url:str, filename:str, pbar):
        """Function that sends the http requests and processes the response.
           param: client - The Session object to send requests.
           type: requests.Session
           param: url - Website base url.
           type: str
           param: filename - File on webserver to request.
           type: str"""
        resp = client.get(url+filename)
        pbar.update(1)
        if resp.status_code in self.codes:
            result = f"/{filename} (Status : {resp.status_code})"
            self.results[resp.status_code].append(filename)
            tqdm.write(result)
            if self.logfile:
                with open(self.logfile,'a') as f:
                    f.write(result+'\n')
    
    def Run(self):
        """The main function of the Pybuster class. This function kicks off and
           controls all of the scanning, the checks on whether the server is down
           or is wildcard matching, and returns a results dictionary that has keys
           that correspond to various http response codes and values that are lists
           of filenames that came back with the corresponding response code."""
        try:
            r = urllib.request.urlopen(self.url+''.join(choice(ascii_letters) for i in range(60)))
            tqdm.write(r.status)
        except urllib.error.HTTPError:
            r = None
        except urllib.error.URLError:
            tqdm.write(f"Do you have the proper address? Because {self.url} seems to be down.")
            sys.exit(1)
        if r and not self.force:
            tqdm.write("Website is wildcard matching. Do you really want to bruteforce this website?")
            sys.exit(0)
        if bool(self.user) and bool(self.password):
            r = requests.post(self.url,auth=HttpNtlmAuth(self.user,self.password))
            if r.status_code != 200:
                tqdm.write("Don't have proper credentials. Please recheck.")
                sys.exit(0)
        if self.logfile:
            with open(self.logfile,'w'):
                pass
        client = requests.Session()
        client.headers['User-Agent'] = self.user_agent
        list_length = len(self.wordlist) * len(self.exts)
        wordlist =  (i+e for i in self.wordlist for e in self.exts)
        clientpool, urllist = (client for i in range(list_length)), (self.url for i in range(list_length))
        pbar = tqdm(total=list_length, leave=False)
        pbars = (pbar for i in range(list_length))
        self._header()
        with concurrent.futures.ThreadPoolExecutor(max_workers=self.threads) as executor:
            try:
                start = time.perf_counter()
                executor.map(self._brute, clientpool, urllist, wordlist, pbars)
            except:
                executor.shutdown()
        self._print_func()
        self._print_func(f"Time elapsed : {time.perf_counter() - start}")
        return self.results

    def _header(self):
        """Output Formatter"""
        self._print_func()
        self._print_func(__version__)
        header_text = "Url:".ljust(20) + self.url +'\n'
        header_text += "Threads:".ljust(20) + str(self.threads) + '\n'
        header_text += "Wordlist:".ljust(20) + self.wordfile + '\n'
        header_text += "Status Codes:".ljust(20) + ','.join(map(str,self.codes)) + '\n'
        header_text += "User Agent:".ljust(20) + self.user_agent + '\n'
        header_text += "Extensions:".ljust(20) + ','.join((i.lstrip('.') for i in self.exts if i != ''))
        self._print_func(header_text)
    
    @utils.pretty
    def _print_func(self, text:str=''):
        end = '\n'
        if text == '':
            end = ''
        tqdm.write(text,end=end)
        



#!/usr/bin/python3
from pydirbuster.main import Pybuster
import argparse
from pydirbuster import utils

parser = argparse.ArgumentParser(description="Python Web Directory and File Brute Forcer")
parser.add_argument('-u', "--url",required=True, help="The url to start brute foroce from.")
parser.add_argument('-w', "--wordlist", dest="wordfile", required=True, help="The wordlist to use for brute force.")
parser.add_argument('--user', help="HTTP Username")
parser.add_argument('--pass', dest="password", help="HTTP Password")
parser.add_argument('-x', dest="exts", type=utils.lister, default=[''], help="File Extensions - must be commad delimited list")
parser.add_argument('-t', "--threads", type=int, default=15, help="The amount of threads to use.")
parser.add_argument('-o', "--output", dest="logfile", help="File to log results.")
parser.add_argument('-s', dest="codes", type=utils.lister, default=[200,204,301,302,307,401,403],
                    help="HTTP Status Codes to accept in a comma delimited list. Default - 200,204,301,302,307,401,403")
parser.add_argument('-f', dest="force", action="store_true", default=False, help="Force wildcard proccessing.")
agent_help = "Custom or random user agent. -z 'User-agent' for custom. -z for random"
parser.add_argument('-z', "--user-agent", dest="user_agent", default=utils.default_user_agent(), nargs='?',
                    action=utils.UserAgentAction, help=agent_help)

args = parser.parse_args()
scanner = Pybuster(**vars(args))
scanner.Run()



#!/usr/bin/python3
import setuptools

def reader(filename:str, lister:bool):
    text = ''
    with open(filename) as f:
        text = f.read()
        if lister:
            text = [ i.split('=')[0] for i in text.splitlines()]
        return text

setuptools.setup(name="pydirbuster",
                 version="0.5",
                 author="Zachary Farquharson",
                 author_email="PercyJackson235@gmail.com",
                 description="Python Web Directory and File Brute Forcer",
                 long_description=reader('README.md', False),
                 long_description_content_type="text/markdown",
                 url="https://github.com/PercyJackson235/pydirbuster",
                 packages=setuptools.find_packages(),
                 classifiers=["Development Status :: 4 - Beta",
                              "Intended Audience :: Developers",
                              "Intended Audience :: Education",
                              "Intended Audience :: End Users/Desktop",
                              "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
                              "Operating System :: OS Independent",
                              "Programming Language :: Python :: 3"],
                 python_requires=">=3",
                 install_requires=reader('requirements.txt', True),
                 scripts=['pydirbuster/pydirbuster'])



"""
pydirbuster Module - Python Web Directory and File Brute Forcer
###############

pydirbuster is module for website file enumeration written in python3.
Basic Usage:
    >>> import pydirbuster
    >>> webbuster = pydirbuster.Pybuster(url="http://doctor/", wordfile="/root/HackTheBox/tools/short.txt",
    ... exts=['php','html'])
    >>> webbuster.Run()
    =================================================================
    Pydirbuster v0.02
    =================================================================
    Url:                http://doctor/
    Threads:            15
    Wordlist:           /root/HackTheBox/tools/short.txt
    Status Codes:       200,204,301,302,307,401,403
    User Agent:         python-requests/2.23.0
    =================================================================
    /.hta (Status : 403)
    /.htaccess.php (Status : 403)
    /.htaccess.html (Status : 403)
    /.hta.html (Status : 403)
    /.htpasswd (Status : 403)
    /.htpasswd.php (Status : 403)
    /.htpasswd.html (Status : 403)
    /.hta.php (Status : 403)
    /.htaccess (Status : 403)
    /index.html (Status : 200)
    =================================================================
    Time elapsed : 1.7652253159903921
    =================================================================

The Pybuster class takes at least a url and a valid wordfile name. The exts parameter
is optional but shown here for verbosity sake and and understanding of common use cases.
All paramters are documented under the Pybuster class.
"""

from . import utils
__version__ = "Pydirbuster v0.05"
from pydirbuster.main import Pybuster
